import Severity from "./vscode/src/vs/base/common/severity.js";
import type { ITextModelContentProvider } from "@codingame/monaco-vscode-model-service-override/vscode/vs/editor/common/services/resolverService";
import { StorageScope, StorageTarget } from "./vscode/src/vs/platform/storage/common/storage.js";
import { type IEditorOverrideServices } from "./vscode/src/vs/editor/standalone/browser/standaloneServices.js";
import { type GetLeadingNonServiceArgs, type ServiceIdentifier, type ServicesAccessor } from "./vscode/src/vs/platform/instantiation/common/instantiation.js";
import type { IAction } from "./vscode/src/vs/base/common/actions.js";
import { type IDisposable } from "./vscode/src/vs/base/common/lifecycle.js";
import type { IWorkbenchConstructionOptions, IWorkspaceProvider } from "@codingame/monaco-vscode-9a6d8b6c-ad4c-5ea3-9558-f43d6dc4c0ad-common/vscode/vs/workbench/browser/web.api";
import { type ILazyWorkbenchContributionInstantiation, type IOnEditorWorkbenchContributionInstantiation, type WorkbenchContributionInstantiation, WorkbenchPhase } from "./vscode/src/vs/workbench/common/contributions.js";
import { IProductService } from "./vscode/src/vs/platform/product/common/productService.service.js";
import type { IConfigurationChangeEvent } from "./vscode/src/vs/platform/configuration/common/configuration.js";
import type { IColorTheme } from "./vscode/src/vs/platform/theme/common/themeService.js";
import { type EnvironmentOverride } from "./workbench.js";
declare global {
    interface Window {
        monacoVscodeApiBuildId?: string;
    }
}
export declare function initialize(overrides: IEditorOverrideServices, container?: HTMLElement, configuration?: IWorkbenchConstructionOptions, env?: EnvironmentOverride): Promise<void>;
export declare function getService<T>(identifier: ServiceIdentifier<T>): Promise<T>;
export declare function createInstance<Ctor extends new (...args: any[]) => any, R extends InstanceType<Ctor>>(ctor: Ctor, ...args: GetLeadingNonServiceArgs<ConstructorParameters<Ctor>>): Promise<R>;
export declare function createInstanceSync<Ctor extends new (...args: any[]) => any, R extends InstanceType<Ctor>>(ctor: Ctor, ...args: GetLeadingNonServiceArgs<ConstructorParameters<Ctor>>): R;
/**
 * Equivalent to `StandaloneServices.withServices` except the callback is called when the services are ready, not just initialized
 */
export declare function withReadyServices(callback: (serviceAccessor: ServicesAccessor) => IDisposable): IDisposable;
export declare const registerWorkbenchContribution: (id: string, contribution: (accessor: ServicesAccessor) => void, instantiation: WorkbenchContributionInstantiation) => void;
export { SyncDescriptor } from "./vscode/src/vs/platform/instantiation/common/descriptors.js";
export { StandaloneServices } from "./vscode/src/vs/editor/standalone/browser/standaloneServices.js";
export { ICommandService } from "./vscode/src/vs/platform/commands/common/commands.service.js";
export { INotificationService } from "./vscode/src/vs/platform/notification/common/notification.service.js";
export { IBulkEditService } from "./vscode/src/vs/editor/browser/services/bulkEditService.service.js";
export { ICodeEditorService } from "./vscode/src/vs/editor/browser/services/codeEditorService.service.js";
export { ILanguageService } from "./vscode/src/vs/editor/common/languages/language.service.js";
export { ILanguageConfigurationService } from "./vscode/src/vs/editor/common/languages/languageConfigurationRegistry.service.js";
export { IEditorWorkerService } from "./vscode/src/vs/editor/common/services/editorWorker.service.js";
export { ILanguageFeaturesService } from "./vscode/src/vs/editor/common/services/languageFeatures.service.js";
export { IModelService } from "./vscode/src/vs/editor/common/services/model.service.js";
export { ITextModelService } from "./vscode/src/vs/editor/common/services/resolverService.service.js";
export { IClipboardService } from "./vscode/src/vs/platform/clipboard/common/clipboardService.service.js";
export { IDialogService, IFileDialogService } from "./vscode/src/vs/platform/dialogs/common/dialogs.service.js";
export { IFileService } from "./vscode/src/vs/platform/files/common/files.service.js";
export { IInstantiationService } from "./vscode/src/vs/platform/instantiation/common/instantiation.js";
export { IMarkerService } from "./vscode/src/vs/platform/markers/common/markers.service.js";
export { IOpenerService } from "./vscode/src/vs/platform/opener/common/opener.service.js";
export { IQuickInputService } from "./vscode/src/vs/platform/quickinput/common/quickInput.service.js";
export { ITelemetryService } from "./vscode/src/vs/platform/telemetry/common/telemetry.service.js";
export { IUriIdentityService } from "./vscode/src/vs/platform/uriIdentity/common/uriIdentity.service.js";
export { IBreadcrumbsService } from "./vscode/src/vs/workbench/browser/parts/editor/breadcrumbs.service.js";
export { IEditorGroupsService } from "./vscode/src/vs/workbench/services/editor/common/editorGroupsService.service.js";
export { GroupDirection, GroupOrientation, GroupLocation, GroupsArrangement, MergeGroupMode, GroupsOrder } from "./vscode/src/vs/workbench/services/editor/common/editorGroupsService.js";
export type { IEditorGroup, IFindGroupScope, GroupLayoutArgument, EditorGroupLayout, IMergeGroupOptions, ICloseEditorOptions, ICloseEditorsFilter, ICloseAllEditorsOptions, IEditorReplacement, IEditorSideGroup } from "./vscode/src/vs/workbench/services/editor/common/editorGroupsService.js";
export { CloseDirection, EditorsOrder } from "./vscode/src/vs/workbench/common/editor.js";
export type { IEditorPane, GroupIdentifier, EditorInputWithOptions, IEditorPartOptions, IEditorPartOptionsChangeEvent, IVisibleEditorPane, IEditorCloseEvent, IUntypedEditorInput, IEditorWillMoveEvent, IEditorWillOpenEvent, IMatchEditorOptions, IActiveEditorChangeEvent, IFindEditorOptions, IEditorControl } from "./vscode/src/vs/workbench/common/editor.js";
export type { IDimension } from "./vscode/src/vs/editor/common/core/dimension.js";
export type { IEditorOptions, ITextEditorOptions, IResourceEditorInput } from "./vscode/src/vs/platform/editor/common/editor.js";
export { EditorInput } from "./vscode/src/vs/workbench/common/editor/editorInput.js";
export type { IGroupModelChangeEvent } from "@codingame/monaco-vscode-ad89fae6-94f0-5ac2-a185-22dea4b68ee0-common/vscode/vs/workbench/common/editor/editorGroupModel";
export { IEditorService } from "./vscode/src/vs/workbench/services/editor/common/editorService.service.js";
export { IEditorResolverService } from "./vscode/src/vs/workbench/services/editor/common/editorResolverService.service.js";
export { ITextEditorService } from "./vscode/src/vs/workbench/services/textfile/common/textEditorService.service.js";
export { IWorkbenchEnvironmentService } from "./vscode/src/vs/workbench/services/environment/common/environmentService.service.js";
export { IHostService } from "./vscode/src/vs/workbench/services/host/browser/host.service.js";
export { ILanguageStatusService } from "./vscode/src/vs/workbench/services/languageStatus/common/languageStatusService.service.js";
export { IPaneCompositePartService } from "./vscode/src/vs/workbench/services/panecomposite/browser/panecomposite.service.js";
export { IPathService } from "./vscode/src/vs/workbench/services/path/common/pathService.service.js";
export { ITextFileService } from "./vscode/src/vs/workbench/services/textfile/common/textfiles.service.js";
export { IWorkingCopyFileService } from "./vscode/src/vs/workbench/services/workingCopy/common/workingCopyFileService.service.js";
export { IConfigurationService } from "./vscode/src/vs/platform/configuration/common/configuration.service.js";
export { IContextKeyService } from "./vscode/src/vs/platform/contextkey/common/contextkey.service.js";
export { IThemeService } from "./vscode/src/vs/platform/theme/common/themeService.service.js";
export { ISnippetsService } from "./vscode/src/vs/workbench/contrib/snippets/browser/snippets.service.js";
export { IWorkspaceContextService } from "./vscode/src/vs/platform/workspace/common/workspace.service.js";
export { IStorageService } from "./vscode/src/vs/platform/storage/common/storage.service.js";
export { IAccessibilitySignalService } from "./vscode/src/vs/platform/accessibilitySignal/browser/accessibilitySignalService.service.js";
export { IDebugService } from "./vscode/src/vs/workbench/contrib/debug/common/debug.service.js";
export { ILoggerService } from "./vscode/src/vs/platform/log/common/log.service.js";
export { LogLevel } from "./vscode/src/vs/platform/log/common/log.js";
export { ILogService } from "./vscode/src/vs/platform/log/common/log.service.js";
export { IViewDescriptorService } from "./vscode/src/vs/workbench/common/views.service.js";
export { IViewsService } from "./vscode/src/vs/workbench/services/views/common/viewsService.service.js";
export { ILayoutService } from "./vscode/src/vs/platform/layout/browser/layoutService.service.js";
export { IPreferencesService } from "./vscode/src/vs/workbench/services/preferences/common/preferences.service.js";
export { IPreferencesSearchService } from "./vscode/src/vs/workbench/contrib/preferences/common/preferences.service.js";
export { IKeybindingEditingService } from "./vscode/src/vs/workbench/services/keybinding/common/keybindingEditing.service.js";
export { IOutputService } from "./vscode/src/vs/workbench/services/output/common/output.service.js";
export { IHistoryService } from "./vscode/src/vs/workbench/services/history/common/history.service.js";
export { GoFilter, GoScope } from "./vscode/src/vs/workbench/services/history/common/history.js";
export { IRemoteAgentService } from "./vscode/src/vs/workbench/services/remote/common/remoteAgentService.service.js";
export { IRemoteAuthorityResolverService } from "./vscode/src/vs/platform/remote/common/remoteAuthorityResolver.service.js";
export { IRemoteSocketFactoryService } from "./vscode/src/vs/platform/remote/common/remoteSocketFactoryService.service.js";
export { IFilesConfigurationService } from "./vscode/src/vs/workbench/services/filesConfiguration/common/filesConfigurationService.service.js";
export { ILabelService } from "./vscode/src/vs/platform/label/common/label.service.js";
export { IWorkbenchLayoutService } from "./vscode/src/vs/workbench/services/layout/browser/layoutService.service.js";
export { ILanguagePackService } from "./vscode/src/vs/platform/languagePacks/common/languagePacks.service.js";
export { ILocaleService } from "./vscode/src/vs/workbench/services/localization/common/locale.service.js";
export { NoOpNotification, NoOpProgress } from "./vscode/src/vs/platform/notification/common/notification.js";
export { NotificationsFilter, NotificationPriority, NeverShowAgainScope } from "./vscode/src/vs/platform/notification/common/notification.js";
export type { NotificationMessage, INotificationProperties, INeverShowAgainOptions, INotification, INotificationActions, INotificationProgress, INotificationProgressProperties, INotificationHandle, IPromptChoice, IPromptChoiceWithMenu, IPromptOptions, IStatusMessageOptions } from "./vscode/src/vs/platform/notification/common/notification.js";
export type { IDialogArgs, DialogType, IDialogResult, IConfirmDialogArgs, IInputDialogArgs, IPromptDialogArgs, IBaseDialogOptions, ICheckbox, ICheckboxResult, IConfirmation, IConfirmationResult, IInput, IInputResult, IInputElement, IPrompt, IPromptResult, IPromptBaseButton, IPromptButton, IPromptCancelButton, IPromptResultWithCancel, IPromptWithCustomCancel, IPromptWithDefaultCancel } from "@codingame/monaco-vscode-2bd6777a-3bff-5b83-ad25-0e029e81ffe9-common/vscode/vs/platform/dialogs/common/dialogs";
export type { IDialogHandler, ICustomDialogOptions, ICustomDialogMarkdown } from "@codingame/monaco-vscode-2bd6777a-3bff-5b83-ad25-0e029e81ffe9-common/vscode/vs/platform/dialogs/common/dialogs";
export type { IMarkdownString, MarkdownStringTrustedOptions } from "./vscode/src/vs/base/common/htmlContent.js";
export { IActivityService } from "./vscode/src/vs/workbench/services/activity/common/activity.service.js";
export type { IBadge } from "@codingame/monaco-vscode-a7c9ae3c-16d2-5d17-86b2-981be7094566-common/vscode/vs/workbench/services/activity/common/activity";
export { IHoverService } from "./vscode/src/vs/platform/hover/browser/hover.service.js";
export { IExplorerService } from "./vscode/src/vs/workbench/contrib/files/browser/files.service.js";
export { IStatusbarService } from "./vscode/src/vs/workbench/services/statusbar/browser/statusbar.service.js";
export { ITitleService } from "./vscode/src/vs/workbench/services/title/browser/titleService.service.js";
export { IBannerService } from "./vscode/src/vs/workbench/services/banner/browser/bannerService.service.js";
export { IWorkspaceTrustManagementService, IWorkspaceTrustRequestService, IWorkspaceTrustEnablementService } from "./vscode/src/vs/platform/workspace/common/workspaceTrust.service.js";
export type { IWorkspaceTrustUriInfo } from "./vscode/src/vs/platform/workspace/common/workspaceTrust.js";
export { IKeybindingService } from "./vscode/src/vs/platform/keybinding/common/keybinding.service.js";
export { ISecretStorageService } from "./vscode/src/vs/platform/secrets/common/secrets.service.js";
export type { ISecretStorageProvider } from "@codingame/monaco-vscode-secret-storage-service-override/vscode/vs/platform/secrets/common/secrets";
export type { IProductConfiguration } from "./vscode/src/vs/base/common/product.js";
export { StorageScope, StorageTarget, Severity, IProductService, WorkbenchPhase };
export type { IAction, IConfigurationChangeEvent, ITextModelContentProvider, IColorTheme, IWorkbenchConstructionOptions, IWorkspaceProvider, IEditorOverrideServices, WorkbenchContributionInstantiation, ILazyWorkbenchContributionInstantiation, IOnEditorWorkbenchContributionInstantiation, ServicesAccessor };
export { ConfigurationTarget } from "./vscode/src/vs/platform/configuration/common/configuration.js";
